@@ -10,6 +10,7 @@
 #include <sstream>
 #include <thread>
 #include <memory>
+#include <regex>
 
 #include <boost/asio.hpp>
 #include <boost/algorithm/string.hpp>
@@ -42,10 +43,11 @@
 
 namespace i2p {
 namespace http {
-	static void LoadExtCSS ()
+
+	static void LoadExtCSS (std::string fileName = "style")
 	{
 		std::stringstream s;
-		std::string styleFile = i2p::fs::DataDirPath ("webconsole/style.css");
+		std::string styleFile = i2p::fs::DataDirPath ("webconsole/"+fileName+".css");
 		if (i2p::fs::Exists(styleFile)) {
 			std::ifstream f(styleFile, std::ifstream::binary);
 			s << f.rdbuf();
@@ -166,6 +168,17 @@ namespace http {
 	static void ShowPageHead (std::stringstream& s)
 	{
 		std::string webroot; i2p::config::GetOption("http.webroot", webroot);
+		std::string theme; i2p::config::GetOption("http.theme", theme);
+
+		const auto isThemeRegex = std::regex("^(white|black|light)");
+		if (!std::regex_search(theme, isThemeRegex))
+		{
+			LoadExtCSS(theme);
+		}
+		else 
+		{
+			LoadExtCSS();
+		}		
 
 		// Page language
 		std::string currLang = i2p::client::context.GetLanguage ()->GetLanguage(); // get current used language
@@ -184,6 +197,17 @@ namespace http {
 			"  <link rel=\"shortcut icon\" href=\"" << itoopieFavicon << "\">\r\n"
 			"  <title>" << tr(/* tr: Webconsole page title */ "Purple I2P Webconsole") << "</title>\r\n";
 		GetStyles(s);
+		if (theme == "black") 
+		{
+		s << 
+		"<style>:root {\r\n"
+         	"--main-bg-color: #242424;\r\n"
+                "--main-text-color: #17ab5c;\r\n"
+                "--main-link-color: #bf64b7;\r\n"
+                "--main-link-hover-color: #000000;\r\n"
+        	"}\r\n</style>";
+		
+		}
 		s <<
 			"</head>\r\n"
 			"<body>\r\n"
@@ -264,9 +288,12 @@ namespace http {
 		s << "<b>" << tr("Uptime") << ":</b> ";
 		ShowUptime(s, i2p::context.GetUptime ());
 		s << "<br>\r\n";
-		s << "<b>" << tr("Network status") << ":</b> ";
-		ShowNetworkStatus (s, i2p::context.GetStatus (), i2p::context.GetTesting(), i2p::context.GetError ());
-		s << "<br>\r\n";
+		if (i2p::context.SupportsV4 () || i2p::context.GetStatus () != eRouterStatusUnknown) // don't show Unknown for ipv6-only
+		{	
+			s << "<b>" << tr("Network status") << ":</b> ";
+			ShowNetworkStatus (s, i2p::context.GetStatus (), i2p::context.GetTesting(), i2p::context.GetError ());
+			s << "<br>\r\n";
+		}	
 		if (i2p::context.SupportsV6 ())
 		{
 			s << "<b>" << tr("Network status v6") << ":</b> ";
@@ -661,7 +688,7 @@ namespace http {
 					else
 					{
 						ls.reset (new i2p::data::LeaseSet2 (storeType));
-						ls->Update (leaseSet->GetBuffer(), leaseSet->GetBufferLen(), false);
+						ls->Update (leaseSet->GetBuffer(), leaseSet->GetBufferLen(), nullptr, false);
 					}
 					if (!ls) return;
 					s << "<div class=\"leaseset listitem";
@@ -947,7 +974,8 @@ namespace http {
 			for (auto& it: sam->GetSessions ())
 			{
 				auto& name = it.second->GetLocalDestination ()->GetNickname ();
-				s << "<div class=\"listitem\"><a href=\"" << webroot << "?page=" << HTTP_PAGE_SAM_SESSION << "&sam_id=" << it.first << "\">";
+				auto sam_id = i2p::data::ByteStreamToBase64 ((const uint8_t *)it.first.data (), it.first.length ()); // base64, becuase session name might be UTF-8
+				s << "<div class=\"listitem\"><a href=\"" << webroot << "?page=" << HTTP_PAGE_SAM_SESSION << "&sam_id=" << sam_id << "\">";
 				s << name << " (" << it.first << ")</a></div>\r\n" << std::endl;
 			}
 			s << "</div>\r\n";
@@ -959,13 +987,26 @@ namespace http {
 	void ShowSAMSession (std::stringstream& s, const std::string& id)
 	{
 		auto sam = i2p::client::context.GetSAMBridge ();
-		if (!sam) {
+		if (!sam) 
+		{
 			ShowError(s, tr("SAM disabled"));
 			return;
 		}
-
-		auto session = sam->FindSession (id);
-		if (!session) {
+		if (id.empty ())
+		{
+			ShowError(s, tr("No sam_id"));
+			return;
+		}		
+		std::vector<uint8_t> sam_id(id.length ()); // id is in base64
+		size_t l = i2p::data::Base64ToByteStream (id, sam_id.data (), sam_id.size ());
+		if (!l)
+		{
+			ShowError(s, tr("Invalid sam_id"));
+			return;
+		}
+		auto session = sam->FindSession ( { (const char *)sam_id.data (), l });
+		if (!session) 
+		{
 			ShowError(s, tr("SAM session not found"));
 			return;
 		}
@@ -977,18 +1018,19 @@ namespace http {
 		s << i2p::client::context.GetAddressBook ().ToAddress(ident) << "</a></div>\r\n";
 		s << "<br>\r\n";
 		s << "<b>" << tr("Streams") << ":</b><br>\r\n<div class=\"list\">\r\n";
-		for (const auto& it: sam->ListSockets(id))
+		for (const auto& it: sam->ListSockets({ (const char *)sam_id.data (), l }))
 		{
 			s << "<div class=\"listitem\">";
 			switch (it->GetSocketType ())
 			{
-				case i2p::client::eSAMSocketTypeSession  : s << "session";  break;
-				case i2p::client::eSAMSocketTypeStream   : s << "stream";   break;
-				case i2p::client::eSAMSocketTypeAcceptor : s << "acceptor"; break;
-				case i2p::client::eSAMSocketTypeForward  : s << "forward";  break;
+				case i2p::client::SAMSocketType::eSAMSocketTypeSession  : s << "session";  break;
+				case i2p::client::SAMSocketType::eSAMSocketTypeStream   : s << "stream";   break;
+				case i2p::client::SAMSocketType::eSAMSocketTypeAcceptor : s << "acceptor"; break;
+				case i2p::client::SAMSocketType::eSAMSocketTypeForward  : s << "forward";  break;
 				default: s << "unknown"; break;
 			}
-			s << " [" << it->GetSocket ().remote_endpoint() << "]";
+			if (it->GetSocketType () != i2p::client::SAMSocketType::eSAMSocketTypeTerminated && it->GetSocket ().is_open ())	
+				s << " [" << it->GetSocket ().remote_endpoint() << "]";
 			s << "</div>\r\n";
 		}
 		s << "</div>\r\n";
@@ -1458,7 +1500,10 @@ namespace http {
 		}
 		else if (cmd == HTTP_COMMAND_RELOAD_CSS)
 		{
-			LoadExtCSS();
+			std::string theme; i2p::config::GetOption("http.theme", theme);
+			
+			if (theme != "light" && theme != "black" && theme !="white") LoadExtCSS(theme);
+			else LoadExtCSS();
 		}
 		else
 		{
