@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2021, The PurpleI2P Project
+* Copyright (c) 2021-2025, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -30,7 +30,10 @@ namespace english // language namespace
 		return n != 1 ? 1 : 0;
 	}
 
-	static std::map<std::string, std::string> strings
+	// Right to Left language?
+	static bool rtl = false;
+
+	static const LocaleStrings strings
 	{
 		{"", ""},
 	};
@@ -42,7 +45,7 @@ namespace english // language namespace
 
 	std::shared_ptr<const i2p::i18n::Locale> GetLocale()
 	{
-		return std::make_shared<i2p::i18n::Locale>(language, strings, plurals, [] (int n)->int { return plural(n); });
+		return std::make_shared<i2p::i18n::Locale>(language, rtl, strings, plurals, [] (int n)->int { return plural(n); });
 	}
 
 } // language
