@@ -18,8 +18,10 @@ namespace i2p
 {
 namespace datagram
 {
-	DatagramDestination::DatagramDestination (std::shared_ptr<i2p::client::ClientDestination> owner, bool gzip):
-		m_Owner (owner), m_DefaultReceiver (nullptr), m_DefaultRawReceiver (nullptr), m_Gzip (gzip)
+	DatagramDestination::DatagramDestination (std::shared_ptr<i2p::client::ClientDestination> owner, 
+	    bool gzip, DatagramVersion version):
+		m_Owner (owner), m_DefaultReceiver (nullptr), m_DefaultRawReceiver (nullptr),
+		m_Gzip (gzip), m_Version (version)
 	{
 		if (m_Gzip)
 			m_Deflator.reset (new i2p::data::GzipDeflator);
@@ -58,25 +60,56 @@ namespace datagram
 	{
 		if (session)
 		{
-			if (m_Owner->GetIdentity ()->GetSigningKeyType () == i2p::data::SIGNING_KEY_TYPE_DSA_SHA1)
+			std::shared_ptr<I2NPMessage>  msg;
+			switch (session->GetVersion ())
 			{
-				uint8_t hash[32];
-				SHA256(payload, len, hash);
-				m_Owner->Sign (hash, 32, m_Signature.data ());
-			}
-			else
-				m_Owner->Sign (payload, len, m_Signature.data ());
+				case eDatagramV3:
+				{
+					constexpr uint8_t flags[] = { 0x00, 0x03 }; // datagram3, no options
+					msg = CreateDataMessage ({{m_Owner->GetIdentity ()->GetIdentHash (), 32}, 
+						{flags, 2}, {payload, len}}, fromPort, toPort, i2p::client::PROTOCOL_TYPE_DATAGRAM3, false); // datagram3
+					break;
+				}
+				case eDatagramV1:
+				{	
+					if (m_Owner->GetIdentity ()->GetSigningKeyType () == i2p::data::SIGNING_KEY_TYPE_DSA_SHA1)
+					{
+						uint8_t hash[32];
+						SHA256(payload, len, hash);
+						m_Owner->Sign (hash, 32, m_Signature.data ());
+					}
+					else
+						m_Owner->Sign (payload, len, m_Signature.data ());
+					msg = CreateDataMessage ({{m_From.data (), m_From.size ()}, {m_Signature.data (), m_Signature.size ()}, {payload, len}},
+						fromPort, toPort, i2p::client::PROTOCOL_TYPE_DATAGRAM, !session->IsRatchets ()); // datagram1
+					break;
+				}
+				case eDatagramV2:
+				{
+					constexpr uint8_t flags[] = { 0x00, 0x02 }; // datagram2, no options
+					// signature
+					std::vector<uint8_t> signedData (len + 32 + 2);
+					memcpy (signedData.data (), m_Owner->GetIdentity ()->GetIdentHash (), 32);
+					memcpy (signedData.data () + 32, flags, 2);
+					memcpy (signedData.data () + 34, payload, len);
+					m_Owner->Sign (signedData.data (), signedData.size (), m_Signature.data ());
+					// TODO: offline signatures and options
+					msg = CreateDataMessage ({{m_From.data (), m_From.size ()}, {flags, 2}, {payload, len}, 
+						{m_Signature.data (), m_Signature.size ()}}, fromPort, toPort, i2p::client::PROTOCOL_TYPE_DATAGRAM2, false); // datagram2
 
-			auto msg = CreateDataMessage ({{m_From.data (), m_From.size ()}, {m_Signature.data (), m_Signature.size ()}, {payload, len}},
-				fromPort, toPort, false, !session->IsRatchets ()); // datagram
-			session->SendMsg(msg);
+					break;
+				}	
+				default:
+					LogPrint (eLogError, "Datagram: datagram type ", (int)session->GetVersion (), " is not supported");
+			}
+			if (msg) session->SendMsg(msg);
 		}
 	}
 
 	void DatagramDestination::SendRawDatagram (std::shared_ptr<DatagramSession> session, const uint8_t * payload, size_t len, uint16_t fromPort, uint16_t toPort)
 	{
 		if (session)
-			session->SendMsg(CreateDataMessage ({{payload, len}}, fromPort, toPort, true, !session->IsRatchets ())); // raw
+			session->SendMsg(CreateDataMessage ({{payload, len}}, fromPort, toPort, i2p::client::PROTOCOL_TYPE_RAW, !session->IsRatchets ())); // raw
 	}
 
 	void DatagramDestination::FlushSendQueue (std::shared_ptr<DatagramSession> session)
@@ -85,26 +118,50 @@ namespace datagram
 			session->FlushSendQueue ();
 	}
 
-	void DatagramDestination::HandleDatagram (uint16_t fromPort, uint16_t toPort,uint8_t * const &buf, size_t len)
+	void DatagramDestination::HandleDatagram (uint16_t fromPort, uint16_t toPort,
+		const uint8_t * buf, size_t len, i2p::garlic::ECIESX25519AEADRatchetSession * from)
 	{
 		i2p::data::IdentityEx identity;
 		size_t identityLen = identity.FromBuffer (buf, len);
+		if (!identityLen) return;
 		const uint8_t * signature = buf + identityLen;
 		size_t headerLen = identityLen + identity.GetSignatureLen ();
 
+		std::shared_ptr<i2p::data::LeaseSet> ls;
 		bool verified = false;
-		if (identity.GetSigningKeyType () == i2p::data::SIGNING_KEY_TYPE_DSA_SHA1)
+		if (from)
 		{
-			uint8_t hash[32];
-			SHA256(buf + headerLen, len - headerLen, hash);
-			verified = identity.Verify (hash, 32, signature);
-		}
-		else
-			verified = identity.Verify (buf + headerLen, len - headerLen, signature);
+			ls = m_Owner->FindLeaseSet (identity.GetIdentHash ());	
+			if (ls)
+			{	
+				uint8_t staticKey[32];
+				ls->Encrypt (nullptr, staticKey);
+				if (!memcmp (from->GetRemoteStaticKey (), staticKey, 32))
+					verified = true;
+				else
+				{	
+					LogPrint (eLogError, "Datagram: Remote LeaseSet static key mismatch for datagram from ", 
+						identity.GetIdentHash ().ToBase32 ());
+					return;
+				}	
+			}	
+		}	
+		if (!verified)
+		{	
+			if (identity.GetSigningKeyType () == i2p::data::SIGNING_KEY_TYPE_DSA_SHA1)
+			{
+				uint8_t hash[32];
+				SHA256(buf + headerLen, len - headerLen, hash);
+				verified = identity.Verify (hash, 32, signature);
+			}
+			else
+				verified = identity.Verify (buf + headerLen, len - headerLen, signature);
+		}	
 
 		if (verified)
 		{
 			auto session = ObtainSession (identity.GetIdentHash());
+			if (ls) session->SetRemoteLeaseSet (ls);
 			session->Ack();
 			auto r = FindReceiver(toPort);
 			if(r)
@@ -126,6 +183,133 @@ namespace datagram
 			LogPrint (eLogWarning, "DatagramDestination: no receiver for raw datagram");
 	}
 
+	void DatagramDestination::HandleDatagram2 (uint16_t fromPort, uint16_t toPort, const uint8_t * buf, size_t len,
+		i2p::garlic::ECIESX25519AEADRatchetSession * from)
+	{
+		if (len < 433)
+		{
+			LogPrint (eLogWarning, "Datagram: datagram2 is too short ", len);
+			return;
+		}
+		i2p::data::IdentityEx identity;
+		size_t identityLen = identity.FromBuffer (buf, len);
+		if (!identityLen) return;
+		size_t signatureLen = identity.GetSignatureLen ();
+		if (signatureLen + identityLen > len) return;
+		
+		std::shared_ptr<i2p::data::LeaseSet> ls;
+		bool verified = false;
+		if (from)
+		{
+			ls = m_Owner->FindLeaseSet (identity.GetIdentHash ());	
+			if (ls)
+			{	
+				uint8_t staticKey[32];
+				ls->Encrypt (nullptr, staticKey);
+				if (!memcmp (from->GetRemoteStaticKey (), staticKey, 32))
+					verified = true;
+				else
+				{	
+					LogPrint (eLogError, "Datagram: Remote LeaseSet static key mismatch for datagram2 from ", 
+						identity.GetIdentHash ().ToBase32 ());
+					return;
+				}	
+			}	
+		}
+		uint16_t flags = bufbe16toh (buf + identityLen);
+		size_t offset = identityLen + 2;
+		if (flags & DATAGRAM2_FLAG_OPTIONS)
+			offset += bufbe16toh (buf + offset) + 2;
+		if (offset > len)
+		{
+			LogPrint (eLogWarning, "Datagram: datagram2 is too short ", len, " expected ", offset);
+			return;
+		}
+		if (!verified)
+		{
+			std::shared_ptr<i2p::crypto::Verifier> transientVerifier;
+			if (flags & DATAGRAM2_FLAG_OFFLINE_SIGNATURE)
+			{	
+				transientVerifier = i2p::data::ProcessOfflineSignature (&identity, buf, len, offset);
+				if (!transientVerifier)
+				{
+					LogPrint (eLogWarning, "Datagram: datagram2 offline signature failed");
+					return;
+				}	
+				signatureLen = transientVerifier->GetSignatureLen ();
+			}	
+			std::vector<uint8_t> signedData (len + 32 - identityLen - signatureLen);
+			memcpy (signedData.data (), identity.GetIdentHash (), 32);
+			memcpy (signedData.data () + 32, buf + identityLen, signedData.size () - 32);
+			verified = transientVerifier ? transientVerifier->Verify (signedData.data (), signedData.size (), buf + len - signatureLen) :
+				identity.Verify (signedData.data (), signedData.size (), buf + len - signatureLen); 
+			if (!verified)
+			{
+				LogPrint (eLogWarning, "Datagram: datagram2 signature verification failed");
+				return;
+			}	
+		}
+		
+		auto session = ObtainSession (identity.GetIdentHash());
+		session->SetVersion (eDatagramV2);
+		if (ls) session->SetRemoteLeaseSet (ls);
+		session->Ack();
+		auto r = FindReceiver(toPort);
+		if(r)
+			r(identity, fromPort, toPort, buf + offset, len - offset - signatureLen);
+		else
+			LogPrint (eLogWarning, "DatagramDestination: no receiver for port ", toPort);
+	}	
+		
+	void DatagramDestination::HandleDatagram3 (uint16_t fromPort, uint16_t toPort, const uint8_t * buf, size_t len,
+		i2p::garlic::ECIESX25519AEADRatchetSession * from)
+	{
+		if (len < 34)
+		{
+			LogPrint (eLogWarning, "Datagram: datagram3 is too short ", len);
+			return;
+		}	
+		if (from)
+		{
+			i2p::data::IdentHash ident(buf);
+			auto ls = m_Owner->FindLeaseSet (ident);	
+			if (ls)
+			{	
+				uint8_t staticKey[32];
+				ls->Encrypt (nullptr, staticKey);
+				if (!memcmp (from->GetRemoteStaticKey (), staticKey, 32))
+				{
+					auto session = ObtainSession (ident);
+					session->SetVersion (eDatagramV3);
+					session->SetRemoteLeaseSet (ls);
+					session->Ack ();
+					auto r = FindReceiver(toPort);
+					if (r)
+					{
+						uint16_t flags = bufbe16toh (buf + 32);
+						size_t offset = 34;
+						if (flags & DATAGRAM3_FLAG_OPTIONS)
+							offset += bufbe16toh (buf + offset) + 2;
+						if (offset > len)
+						{
+							LogPrint (eLogWarning, "Datagram: datagram3 is too short ", len, " expected ", offset);
+							return;
+						}	
+						r(*ls->GetIdentity (), fromPort, toPort, buf + offset, len - offset);
+					}	
+					else
+						LogPrint (eLogWarning, "Datagram: no receiver for port ", toPort);
+				}	
+				else
+					LogPrint (eLogError, "Datagram: Remote LeaseSet static key mismatch for datagram3 from ", ident.ToBase32 ());
+			}
+			else
+				LogPrint (eLogError, "Datagram: No remote LeaseSet for ", ident.ToBase32 ());
+		}
+		else
+			LogPrint (eLogInfo, "Datagram: datagram3 received from non-ratchets session");
+	}	
+		
 	void DatagramDestination::SetReceiver (const Receiver& receiver, uint16_t port)
 	{
 		std::lock_guard<std::mutex> lock(m_ReceiversMutex);
@@ -194,17 +378,31 @@ namespace datagram
 		return r;
 	}
 
-	void DatagramDestination::HandleDataMessagePayload (uint16_t fromPort, uint16_t toPort, const uint8_t * buf, size_t len, bool isRaw)
+	void DatagramDestination::HandleDataMessagePayload (uint16_t fromPort, uint16_t toPort, 
+		const uint8_t * buf, size_t len, uint8_t protocolType, i2p::garlic::ECIESX25519AEADRatchetSession * from)
 	{
 		// unzip it
 		uint8_t uncompressed[MAX_DATAGRAM_SIZE];
 		size_t uncompressedLen = m_Inflator.Inflate (buf, len, uncompressed, MAX_DATAGRAM_SIZE);
 		if (uncompressedLen)
 		{
-			if (isRaw)
-				HandleRawDatagram (fromPort, toPort, uncompressed, uncompressedLen);
-			else
-				HandleDatagram (fromPort, toPort, uncompressed, uncompressedLen);
+			switch (protocolType)
+			{	
+			 	case i2p::client::PROTOCOL_TYPE_RAW:
+					HandleRawDatagram (fromPort, toPort, uncompressed, uncompressedLen);
+				break;
+				case i2p::client::PROTOCOL_TYPE_DATAGRAM3:
+					HandleDatagram3 (fromPort, toPort, uncompressed, uncompressedLen, from);	
+				break;
+				case i2p::client::PROTOCOL_TYPE_DATAGRAM:
+					HandleDatagram (fromPort, toPort, uncompressed, uncompressedLen, from);
+				break;
+				case i2p::client::PROTOCOL_TYPE_DATAGRAM2:
+					HandleDatagram2 (fromPort, toPort, uncompressed, uncompressedLen, from);
+				break;
+				default:
+					LogPrint (eLogInfo, "Datagram: unknown protocol type ", protocolType);
+			};
 		}
 		else
 			LogPrint (eLogWarning, "Datagram: decompression failed");
@@ -212,7 +410,7 @@ namespace datagram
 
 	std::shared_ptr<I2NPMessage> DatagramDestination::CreateDataMessage (
 		const std::vector<std::pair<const uint8_t *, size_t> >& payloads,
-		uint16_t fromPort, uint16_t toPort, bool isRaw, bool checksum)
+		uint16_t fromPort, uint16_t toPort, uint8_t protocolType, bool checksum)
 	{
 		size_t size;
 		auto msg = m_I2NPMsgsPool.AcquireShared ();
@@ -228,8 +426,8 @@ namespace datagram
 		{
 			htobe32buf (msg->GetPayload (), size); // length
 			htobe16buf (buf + 4, fromPort); // source port
-			htobe16buf (buf + 6, toPort); // destination port
-			buf[9] = isRaw ? i2p::client::PROTOCOL_TYPE_RAW : i2p::client::PROTOCOL_TYPE_DATAGRAM; // raw or datagram protocol
+			htobe16buf (buf + 6, toPort); // destination port	
+			buf[9] = protocolType; // raw or datagram protocol
 			msg->len += size + 4;
 			msg->FillI2NPMessageHeader (eI2NPData, 0, checksum);
 		}
@@ -264,14 +462,16 @@ namespace datagram
 		std::shared_ptr<DatagramSession> session = nullptr;
 		std::lock_guard<std::mutex> lock(m_SessionsMutex);
 		auto itr = m_Sessions.find(identity);
-		if (itr == m_Sessions.end()) {
+		if (itr == m_Sessions.end()) 
+		{
 			// not found, create new session
 			session = std::make_shared<DatagramSession>(m_Owner, identity);
+			session->SetVersion (m_Version);
 			session->Start ();
-			m_Sessions[identity] = session;
-		} else {
+			m_Sessions.emplace (identity, session);
+		} 
+		else
 			session = itr->second;
-		}
 		return session;
 	}
 
@@ -288,8 +488,7 @@ namespace datagram
 	DatagramSession::DatagramSession(std::shared_ptr<i2p::client::ClientDestination> localDestination,
 		const i2p::data::IdentHash & remoteIdent) :
 		m_LocalDestination(localDestination), m_RemoteIdent(remoteIdent),
-		m_LastUse (0), m_LastFlush (0),
-		m_RequestingLS(false)
+		m_LastUse (0), m_LastFlush (0), m_RequestingLS (false), m_Version (eDatagramV1)
 	{
 	}
 
