@@ -88,16 +88,21 @@ namespace stream
 		m_TunnelsChangeSequenceNumber (0), m_LastReceivedSequenceNumber (-1), m_PreviousReceivedSequenceNumber (-1),
 		m_LastConfirmedReceivedSequenceNumber (0), // for limit inbound speed
 		m_Status (eStreamStatusNew), m_IsIncoming (false), m_IsAckSendScheduled (false), m_IsNAcked (false), m_IsFirstACK (false), 
-		m_IsResendNeeded (false), m_IsFirstRttSample (false), m_IsSendTime (true), m_IsWinDropped (true), m_IsClientChoked (false),
-		m_IsTimeOutResend (false), m_IsImmediateAckRequested (false), m_IsRemoteLeaseChangeInProgress (false), m_IsBufferEmpty (false), m_LocalDestination (local),
-		m_RemoteLeaseSet (remote), m_ReceiveTimer (m_Service), m_SendTimer (m_Service), m_ResendTimer (m_Service),
-		m_AckSendTimer (m_Service), m_NumSentBytes (0), m_NumReceivedBytes (0), m_Port (port),
-		m_RTT (INITIAL_RTT), m_MinRTT (INITIAL_RTT), m_SlowRTT (INITIAL_RTT), m_FastRTT (INITIAL_RTT), m_WindowSize (INITIAL_WINDOW_SIZE), m_LastWindowDropSize  (0),
+		m_IsResendNeeded (false), m_IsFirstRttSample (false), m_IsSendTime (true), 
+		m_IsWinDropped (true), m_IsChoking2 (false), m_IsClientChoked (false), m_IsClientChoked2 (false),
+		m_IsTimeOutResend (false), m_IsImmediateAckRequested (false), m_IsRemoteLeaseChangeInProgress (false), 
+		m_IsBufferEmpty (false), m_IsJavaClient (false), m_DontSign (local.GetOwner ()->IsStreamingDontSign ()), 
+		m_LocalDestination (local), m_RemoteLeaseSet (remote), m_ReceiveTimer (m_Service), 
+		m_SendTimer (m_Service), m_ResendTimer (m_Service), m_AckSendTimer (m_Service), m_NumSentBytes (0),
+		m_NumReceivedBytes (0), m_Port (port), m_RTT (INITIAL_RTT), m_MinRTT (INITIAL_RTT), 
+		m_SlowRTT (INITIAL_RTT), m_FastRTT (INITIAL_RTT), m_WindowSize (INITIAL_WINDOW_SIZE),
+		m_MaxWindowSize (local.GetOwner ()->GetStreamingMaxWindowSize ()), m_LastWindowDropSize (0), 
 		m_WindowDropTargetSize (0), m_WindowIncCounter (0), m_RTO (INITIAL_RTO),
 		m_AckDelay (local.GetOwner ()->GetStreamingAckDelay ()), m_PrevRTTSample (INITIAL_RTT), 
-		m_Jitter (0), m_MinPacingTime (0),
-		m_PacingTime (INITIAL_PACING_TIME), m_PacingTimeRem (0), m_LastSendTime (0), m_LastACKRecieveTime (0), m_ACKRecieveInterval (local.GetOwner ()->GetStreamingAckDelay ()), m_RemoteLeaseChangeTime (0), m_LastWindowIncTime (0),
-		m_LastACKSendTime (0), m_PacketACKInterval (1), m_PacketACKIntervalRem (0), // for limit inbound speed
+		m_Jitter (0), m_MinPacingTime (0), m_PacingTime (INITIAL_PACING_TIME), m_PacingTimeRem (0), 
+		m_LastSendTime (0), m_LastACKRecieveTime (0), m_ACKRecieveInterval (local.GetOwner ()->GetStreamingAckDelay ()), 
+		m_RemoteLeaseChangeTime (0), m_LastWindowIncTime (0), m_LastACKRequestTime (0), m_LastACKSendTime (0), 
+		m_PacketACKInterval (1), m_PacketACKIntervalRem (0), // for limit inbound speed
 		m_NumResendAttempts (0), m_NumPacketsToSend (0), m_JitterAccum (0), m_JitterDiv (1), m_MTU (STREAMING_MTU)
 	{
 		RAND_bytes ((uint8_t *)&m_RecvStreamID, 4);
@@ -116,14 +121,19 @@ namespace stream
 		m_TunnelsChangeSequenceNumber (0), m_LastReceivedSequenceNumber (-1), m_PreviousReceivedSequenceNumber (-1),
 		m_LastConfirmedReceivedSequenceNumber (0), // for limit inbound speed
 		m_Status (eStreamStatusNew), m_IsIncoming (true), m_IsAckSendScheduled (false), m_IsNAcked (false), m_IsFirstACK (false),  
-		m_IsResendNeeded (false), m_IsFirstRttSample (false), m_IsSendTime (true), m_IsWinDropped (true), m_IsClientChoked (false),
-		m_IsTimeOutResend (false), m_IsImmediateAckRequested (false), m_IsRemoteLeaseChangeInProgress (false), m_IsBufferEmpty (false), m_LocalDestination (local),
-		m_ReceiveTimer (m_Service), m_SendTimer (m_Service), m_ResendTimer (m_Service), m_AckSendTimer (m_Service),
-		m_NumSentBytes (0), m_NumReceivedBytes (0), m_Port (0), m_RTT (INITIAL_RTT), m_MinRTT (INITIAL_RTT), m_SlowRTT (INITIAL_RTT), m_FastRTT (INITIAL_RTT),
-		m_WindowSize (INITIAL_WINDOW_SIZE), m_LastWindowDropSize  (0), m_WindowDropTargetSize (0), m_WindowIncCounter (0), 
-		m_RTO (INITIAL_RTO), m_AckDelay (local.GetOwner ()->GetStreamingAckDelay ()),
-		m_PrevRTTSample (INITIAL_RTT), m_Jitter (0), m_MinPacingTime (0),
-		m_PacingTime (INITIAL_PACING_TIME), m_PacingTimeRem (0), m_LastSendTime (0), m_LastACKRecieveTime (0), m_ACKRecieveInterval (local.GetOwner ()->GetStreamingAckDelay ()), m_RemoteLeaseChangeTime (0), m_LastWindowIncTime (0),
+		m_IsResendNeeded (false), m_IsFirstRttSample (false), m_IsSendTime (true), 
+		m_IsWinDropped (true), m_IsChoking2 (false), m_IsClientChoked (false), m_IsClientChoked2 (false),
+		m_IsTimeOutResend (false), m_IsImmediateAckRequested (false), m_IsRemoteLeaseChangeInProgress (false),
+		m_IsBufferEmpty (false), m_IsJavaClient (false), m_DontSign (local.GetOwner ()->IsStreamingDontSign ()),
+		m_LocalDestination (local),m_ReceiveTimer (m_Service), m_SendTimer (m_Service),
+		m_ResendTimer (m_Service), m_AckSendTimer (m_Service),m_NumSentBytes (0), m_NumReceivedBytes (0),
+		m_Port (0), m_RTT (INITIAL_RTT), m_MinRTT (INITIAL_RTT), m_SlowRTT (INITIAL_RTT), m_FastRTT (INITIAL_RTT),
+		m_WindowSize (INITIAL_WINDOW_SIZE), m_MaxWindowSize (local.GetOwner ()->GetStreamingMaxWindowSize ()), 
+		m_LastWindowDropSize  (0), m_WindowDropTargetSize (0), m_WindowIncCounter (0), m_RTO (INITIAL_RTO),
+		m_AckDelay (local.GetOwner ()->GetStreamingAckDelay ()),m_PrevRTTSample (INITIAL_RTT), m_Jitter (0), 
+		m_MinPacingTime (0), m_PacingTime (INITIAL_PACING_TIME), m_PacingTimeRem (0), m_LastSendTime (0),
+		m_LastACKRecieveTime (0), m_ACKRecieveInterval (local.GetOwner ()->GetStreamingAckDelay ()), 
+		m_RemoteLeaseChangeTime (0), m_LastWindowIncTime (0), m_LastACKRequestTime (0),
 		m_LastACKSendTime (0), m_PacketACKInterval (1), m_PacketACKIntervalRem (0), // for limit inbound speed
 		m_NumResendAttempts (0), m_NumPacketsToSend (0), m_JitterAccum (0), m_JitterDiv (1), m_MTU (STREAMING_MTU)
 	{
@@ -156,7 +166,16 @@ namespace stream
 	}
 
 	void Stream::CleanUp ()
-	{
+	{		
+		if (m_RoutingSession && !m_SentPackets.empty ()) // free up space in shared window
+		{
+			int numPackets = m_SentPackets.size ();
+			int numSentPackets = m_RoutingSession->NumSentPackets ();
+			numSentPackets -= numPackets;
+			if (numSentPackets < 0) numSentPackets = 0;
+			m_RoutingSession->SetNumSentPackets (numSentPackets);
+		}
+		
 		m_SendBuffer.CleanUp ();
 		while (!m_ReceiveQueue.empty ())
 		{
@@ -187,7 +206,7 @@ namespace stream
 		if (!m_SendStreamID)
 		{	
 			m_SendStreamID = packet->GetReceiveStreamID ();
-			if (!m_RemoteIdentity && packet->GetNACKCount () == 8 && // first incoming packet
+			if (!m_RemoteIdentity && !packet->from && packet->GetNACKCount () == 8 && // first incoming packet
 			    memcmp (packet->GetNACKs (), m_LocalDestination.GetOwner ()->GetIdentHash (), 32))
 			{
 				LogPrint (eLogWarning, "Streaming: Destination mismatch for ", m_LocalDestination.GetOwner ()->GetIdentHash ().ToBase32 ());
@@ -288,8 +307,41 @@ namespace stream
 			else
 			{
 				LogPrint (eLogInfo, "Streaming: Missing messages on sSID=", m_SendStreamID, ": from ", m_LastReceivedSequenceNumber + 1, " to ", receivedSeqn - 1);
-				// save message and wait for missing message again
-				SavePacket (packet);
+				if ((receivedSeqn - m_LastReceivedSequenceNumber) >= m_MaxWindowSize*3)
+				{
+					m_LocalDestination.DeletePacket (packet);
+					m_IsChoking2 = true;
+				}
+				else if (m_SavedPackets.empty () && (receivedSeqn - m_LastReceivedSequenceNumber) >= 256)
+				{
+					m_LocalDestination.DeletePacket (packet);
+					m_IsChoking2 = true;
+				}
+				else if (!m_SavedPackets.empty ())
+				{
+					uint8_t numNacks = 0;
+					auto lastSavedSeq = 0;
+					auto nextSeqn = m_LastReceivedSequenceNumber + 1;
+					for (auto it: m_SavedPackets)
+					{
+						auto seqn = it->GetSeqn ();
+						if ((int)seqn > lastSavedSeq) lastSavedSeq = seqn;
+						for (uint32_t i = nextSeqn; i < seqn; i++) numNacks++;
+						nextSeqn = seqn + 1;
+					}
+				
+					if (numNacks + (receivedSeqn - lastSavedSeq) >= 256)
+					{
+						m_LocalDestination.DeletePacket (packet);
+						m_IsChoking2 = true;
+					}
+					else
+						// save message and wait for missing message again
+						SavePacket (packet);
+				}
+				else
+					// save message and wait for missing message again
+					SavePacket (packet);
 				if (m_LastReceivedSequenceNumber >= 0)
 				{
 					if (!m_IsAckSendScheduled)
@@ -380,15 +432,27 @@ namespace stream
 				}
 				if (delayRequested >= DELAY_CHOKING)
 				{
-					if (!m_IsClientChoked)
+					if (delayRequested == 65535)
+					{
+						m_IsClientChoked2 = true;
+						m_DropWindowDelaySequenceNumber = m_SequenceNumber-1;
+					}
+					else if (!m_IsClientChoked)
 					{
 						LogPrint (eLogDebug, "Streaming: Client choked, set min. window size");
+						if (delayRequested == DELAY_CHOKING_JAVA) // java detected
+						{
+							LogPrint (eLogDebug, "Streaming: limit window size for java client");
+							m_MaxWindowSize = 64;
+							m_IsJavaClient = true;
+							if (m_RoutingSession) m_RoutingSession->SetIsWithJava (true);
+						}
 						m_WindowDropTargetSize = MIN_WINDOW_SIZE;
 						m_LastWindowDropSize = 0;
 						m_WindowIncCounter = 0;
 						m_IsClientChoked = true;
 						m_IsWinDropped = false;
-						m_DropWindowDelaySequenceNumber = m_SequenceNumber;
+						m_DropWindowDelaySequenceNumber = m_SequenceNumber-1;
 						m_IsFirstRttSample = true;
 						UpdatePacingTime ();
 					}
@@ -397,8 +461,10 @@ namespace stream
 			optionData += 2;
 		}
 
+		bool verified = true;
 		if (flags & PACKET_FLAG_FROM_INCLUDED)
 		{
+			verified = false;
 			if (m_RemoteLeaseSet) m_RemoteIdentity = m_RemoteLeaseSet->GetIdentity ();
 			if (!m_RemoteIdentity)
 				m_RemoteIdentity = std::make_shared<i2p::data::IdentityEx>(optionData, optionSize);
@@ -409,7 +475,26 @@ namespace stream
 			}
 			optionData += m_RemoteIdentity->GetFullLen ();
 			if (!m_RemoteLeaseSet)
-				LogPrint (eLogDebug, "Streaming: Incoming stream from ", m_RemoteIdentity->GetIdentHash ().ToBase64 (), ", sSID=", m_SendStreamID, ", rSID=", m_RecvStreamID);
+			{	
+				LogPrint (eLogDebug, "Streaming: Incoming stream from ", m_RemoteIdentity->GetIdentHash ().ToBase32 (), ", sSID=", m_SendStreamID, ", rSID=", m_RecvStreamID);
+				if (packet->from) // try to obtain LeaseSet if came from ratchets session	
+					m_RemoteLeaseSet = m_LocalDestination.GetOwner ()->FindLeaseSet (m_RemoteIdentity->GetIdentHash ());
+			}	
+			if (packet->from && m_RemoteLeaseSet)
+			{
+				// stream came from ratchets session and static key must match one from LeaseSet
+				uint8_t staticKey[32];
+				m_RemoteLeaseSet->Encrypt (nullptr, staticKey);
+				if (memcmp (packet->from->GetRemoteStaticKey (), staticKey, 32))
+				{
+					LogPrint (eLogError, "Streaming: Remote LeaseSet static key mismatch for stream from ", 
+						m_RemoteIdentity->GetIdentHash ().ToBase32 ());
+					return false;
+				}	
+				verified = true;
+				if (!(flags & PACKET_FLAG_SIGNATURE_INCLUDED))
+					m_DontSign = true; // don't sign if the remote didn't sign
+			}
 		}
 
 		if (flags & PACKET_FLAG_MAX_PACKET_SIZE_INCLUDED)
@@ -419,6 +504,30 @@ namespace stream
 			optionData += 2;
 		}
 
+		if (flags & (PACKET_FLAG_CLOSE | PACKET_FLAG_RESET))
+		{
+			verified = false;
+			if (packet->from)
+			{	
+				if (!m_RemoteLeaseSet && m_RemoteIdentity)
+					m_RemoteLeaseSet = m_LocalDestination.GetOwner ()->FindLeaseSet (m_RemoteIdentity->GetIdentHash ());
+				if (m_RemoteLeaseSet)
+				{
+					uint8_t staticKey[32];
+					m_RemoteLeaseSet->Encrypt (nullptr, staticKey);
+					if (memcmp (packet->from->GetRemoteStaticKey (), staticKey, 32))
+					{
+						LogPrint (eLogError, "Streaming: Remote LeaseSet static key mismatch for stream from ", 
+							m_RemoteIdentity->GetIdentHash ().ToBase32 ());
+						return false;
+					}	
+					verified = true;
+				}	
+				else // invalid stream, safe to close
+					verified = true;
+			}
+		}	
+		
 		if (flags & PACKET_FLAG_OFFLINE_SIGNATURE)
 		{
 			if (!m_RemoteIdentity)
@@ -426,68 +535,93 @@ namespace stream
 				LogPrint (eLogInfo, "Streaming: offline signature without identity");
 				return false;
 			}
-			// if we have it in LeaseSet already we don't need to parse it again
-			if (m_RemoteLeaseSet) m_TransientVerifier = m_RemoteLeaseSet->GetTransientVerifier ();
-			if (m_TransientVerifier)
+			if (verified)
 			{
-				// skip option data
-				optionData += 6; // timestamp and key type
-				optionData += m_TransientVerifier->GetPublicKeyLen (); // public key
+				// skip offline signature
+				optionData += 4; // timestamp 
+				uint16_t keyType = bufbe16toh (optionData); optionData += 2; // key type
+				std::unique_ptr<i2p::crypto::Verifier> transientVerifier (i2p::data::IdentityEx::CreateVerifier (keyType));
+				if (!transientVerifier)
+				{
+					LogPrint (eLogInfo, "Streaming: Unknown offline signature key type ", (int)keyType);
+					return false;
+				}
+				optionData += transientVerifier->GetPublicKeyLen (); // public key
 				optionData += m_RemoteIdentity->GetSignatureLen (); // signature
 			}
 			else
-			{
-				// transient key
-				size_t offset = 0;
-				m_TransientVerifier = i2p::data::ProcessOfflineSignature (m_RemoteIdentity, optionData, optionSize - (optionData - packet->GetOptionData ()), offset);
-				optionData += offset;
-				if (!m_TransientVerifier)
+			{	
+				// if we have it in LeaseSet already we don't need to parse it again
+				if (m_RemoteLeaseSet) m_TransientVerifier = m_RemoteLeaseSet->GetTransientVerifier ();
+				if (m_TransientVerifier)
 				{
-					LogPrint (eLogError, "Streaming: offline signature failed");
-					return false;
+					// skip option data
+					optionData += 6; // timestamp and key type
+					optionData += m_TransientVerifier->GetPublicKeyLen (); // public key
+					optionData += m_RemoteIdentity->GetSignatureLen (); // signature
 				}
-			}
+				else
+				{
+					// transient key
+					size_t offset = 0;
+					m_TransientVerifier = i2p::data::ProcessOfflineSignature (m_RemoteIdentity, optionData, optionSize - (optionData - packet->GetOptionData ()), offset);
+					optionData += offset;
+					if (!m_TransientVerifier)
+					{
+						LogPrint (eLogError, "Streaming: offline signature failed");
+						return false;
+					}
+				}
+			}	
 		}
 
 		if (flags & PACKET_FLAG_SIGNATURE_INCLUDED)
 		{
-			bool verified = false;
 			auto signatureLen = m_TransientVerifier ? m_TransientVerifier->GetSignatureLen () : m_RemoteIdentity->GetSignatureLen ();
 			if (signatureLen > packet->GetLength ())
 			{
 				LogPrint (eLogError, "Streaming: Signature too big, ", signatureLen, " bytes");
 				return false;
 			}	
-			if(signatureLen <= 256)
-			{
-				// standard
-				uint8_t signature[256];
-				memcpy (signature, optionData, signatureLen);
-				memset (const_cast<uint8_t *>(optionData), 0, signatureLen);
-				verified = m_TransientVerifier ?
-					m_TransientVerifier->Verify (packet->GetBuffer (), packet->GetLength (), signature) :
-					m_RemoteIdentity->Verify (packet->GetBuffer (), packet->GetLength (), signature);
-				if (verified)
-					memcpy (const_cast<uint8_t *>(optionData), signature, signatureLen);
-			}
-			else
-			{
-				// post quantum
-				std::vector<uint8_t> signature(signatureLen);
-				memcpy (signature.data (), optionData, signatureLen);
-				memset (const_cast<uint8_t *>(optionData), 0, signatureLen);
-				verified = m_TransientVerifier ?
-					m_TransientVerifier->Verify (packet->GetBuffer (), packet->GetLength (), signature.data ()) :
-					m_RemoteIdentity->Verify (packet->GetBuffer (), packet->GetLength (), signature.data ());
-			}
+			if (!verified) // packet was not verified through session
+			{	
+				// verify actual signature
+				if (signatureLen <= 256)
+				{
+					// standard
+					uint8_t signature[256];
+					memcpy (signature, optionData, signatureLen);
+					memset (const_cast<uint8_t *>(optionData), 0, signatureLen);
+					verified = m_TransientVerifier ?
+						m_TransientVerifier->Verify (packet->GetBuffer (), packet->GetLength (), signature) :
+						m_RemoteIdentity->Verify (packet->GetBuffer (), packet->GetLength (), signature);
+					if (verified)
+						memcpy (const_cast<uint8_t *>(optionData), signature, signatureLen);
+				}
+				else
+				{
+					// post quantum
+					std::vector<uint8_t> signature(signatureLen);
+					memcpy (signature.data (), optionData, signatureLen);
+					memset (const_cast<uint8_t *>(optionData), 0, signatureLen);
+					verified = m_TransientVerifier ?
+						m_TransientVerifier->Verify (packet->GetBuffer (), packet->GetLength (), signature.data ()) :
+						m_RemoteIdentity->Verify (packet->GetBuffer (), packet->GetLength (), signature.data ());
+				}
+			}	
 			if (verified)
 				optionData += signatureLen;
 			else
 			{
 				LogPrint (eLogError, "Streaming: Signature verification failed, sSID=", m_SendStreamID, ", rSID=", m_RecvStreamID);
 				return false;
-			}	
+			}
 		}
+		if (!verified)
+		{
+			LogPrint (eLogError, "Streaming: Missing signature, sSID=", m_SendStreamID, ", rSID=", m_RecvStreamID);
+			return false;
+		}	
 		if (immediateAckRequested)
 			SendQuickAck ();
 		return true;
@@ -528,6 +662,7 @@ namespace stream
 		}
 		int rttSample = INT_MAX;
 		int incCounter = 0;
+		int ackPacketsCounter = 0;
 		m_IsNAcked = false;
 		m_IsResendNeeded = false;
 		int nackCount = packet->GetNACKCount ();
@@ -569,7 +704,8 @@ namespace stream
 				m_SentPackets.erase (it++);
 				m_LocalDestination.DeletePacket (sentPacket);
 				acknowledged = true;
-				if (m_WindowIncCounter < MAX_WINDOW_SIZE && !m_IsFirstACK && !m_IsWinDropped)
+				ackPacketsCounter++;
+				if (m_WindowIncCounter < m_MaxWindowSize && !m_IsFirstACK && !m_IsWinDropped)
 					incCounter++;
 			}
 			else
@@ -651,6 +787,8 @@ namespace stream
 		}
 		if (m_IsClientChoked && (ackThrough >= m_DropWindowDelaySequenceNumber || m_SentPackets.empty ()))
 			m_IsClientChoked = false;
+		if (m_IsClientChoked2 && (ackThrough >= m_DropWindowDelaySequenceNumber || m_SentPackets.empty ()))
+			m_IsClientChoked2 = false;
 		if (m_IsWinDropped && ackThrough > m_DropWindowDelaySequenceNumber)
 		{
 			m_IsFirstRttSample = true;
@@ -687,6 +825,13 @@ namespace stream
 		}
 		if (acknowledged)
 		{
+			if (m_RoutingSession)
+			{
+				int numSentPackets = m_RoutingSession->NumSentPackets ();
+				numSentPackets -= ackPacketsCounter;
+				if (numSentPackets < 0) numSentPackets = 0;
+				m_RoutingSession->SetNumSentPackets (numSentPackets);
+			}
 			m_NumResendAttempts = 0;
 			m_IsTimeOutResend = false;
 			SendBuffer ();
@@ -774,6 +919,20 @@ namespace stream
 		}
 		else if (numMsgs > m_NumPacketsToSend)
 			numMsgs = m_NumPacketsToSend;
+		if (m_RoutingSession)
+		{
+			m_IsJavaClient = m_RoutingSession->IsWithJava ();
+			if (m_IsJavaClient) m_MaxWindowSize = 64;
+			int numSentPackets = m_RoutingSession->NumSentPackets ();
+			int numPacketsToSend = m_MaxWindowSize - numSentPackets;
+			if (numPacketsToSend <= 0) // shared window is full
+			{
+				m_LastSendTime = ts;
+				return;
+			}
+			else if (numMsgs > numPacketsToSend)
+				numMsgs = numPacketsToSend;
+		}
 		bool isNoAck = m_LastReceivedSequenceNumber < 0; // first packet
 		std::vector<Packet *> packets;
 		while ((m_Status == eStreamStatusNew) || (IsEstablished () && !m_SendBuffer.IsEmpty () && numMsgs > 0))
@@ -796,10 +955,18 @@ namespace stream
 			if (m_Status == eStreamStatusNew && !m_SendStreamID && m_RemoteIdentity)
 			{
 				// first SYN packet
-				packet[size] = 8;
-				size++; // NACK count
-				memcpy (packet + size, m_RemoteIdentity->GetIdentHash (), 32);
-				size += 32;
+				if (m_DontSign)
+				{
+					// remote ident is useless without signature, don't include it
+					packet[size] = 0; size++; // NACK count
+				}
+				else
+				{	
+					packet[size] = 8;
+					size++; // NACK count
+					memcpy (packet + size, m_RemoteIdentity->GetIdentHash (), 32);
+					size += 32;
+				}	
 			}
 			else
 			{	
@@ -812,14 +979,14 @@ namespace stream
 			{
 				// initial packet
 				m_Status = eStreamStatusOpen;
-				if (!m_RemoteLeaseSet) m_RemoteLeaseSet = m_LocalDestination.GetOwner ()->FindLeaseSet (m_RemoteIdentity->GetIdentHash ());;
+				if (!m_RemoteLeaseSet) m_RemoteLeaseSet = m_LocalDestination.GetOwner ()->FindLeaseSet (m_RemoteIdentity->GetIdentHash ());
 				if (m_RemoteLeaseSet)
 				{
 					m_RoutingSession = m_LocalDestination.GetOwner ()->GetRoutingSession (m_RemoteLeaseSet, true, !m_IsIncoming);
 					m_MTU = (m_RoutingSession && m_RoutingSession->IsRatchets ()) ? STREAMING_MTU_RATCHETS : STREAMING_MTU;
 				}
-				uint16_t flags = PACKET_FLAG_SYNCHRONIZE | PACKET_FLAG_FROM_INCLUDED |
-					PACKET_FLAG_SIGNATURE_INCLUDED | PACKET_FLAG_MAX_PACKET_SIZE_INCLUDED;
+				uint16_t flags = PACKET_FLAG_SYNCHRONIZE | PACKET_FLAG_FROM_INCLUDED | PACKET_FLAG_MAX_PACKET_SIZE_INCLUDED;
+				if (!m_DontSign) flags |= PACKET_FLAG_SIGNATURE_INCLUDED;
 				if (isNoAck) flags |= PACKET_FLAG_NO_ACK;
 				bool isOfflineSignature = m_LocalDestination.GetOwner ()->GetPrivateKeys ().IsOfflineSignature ();
 				if (isOfflineSignature) flags |= PACKET_FLAG_OFFLINE_SIGNATURE;
@@ -833,25 +1000,45 @@ namespace stream
 				size += identityLen; // from
 				htobe16buf (packet + size, m_MTU);
 				size += 2; // max packet size
-				if (isOfflineSignature)
+				if (m_DontSign)
 				{
-					const auto& offlineSignature = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetOfflineSignature ();
-					memcpy (packet + size, offlineSignature.data (), offlineSignature.size ());
-					size += offlineSignature.size (); // offline signature
+					htobe16buf (optionsSize, packet + size - 2 - optionsSize); // actual options size
+					size += m_SendBuffer.Get (packet + size, m_MTU); // payload
 				}
-				uint8_t * signature = packet + size; // set it later
-				memset (signature, 0, signatureLen); // zeroes for now
-				size += signatureLen; // signature
-				htobe16buf (optionsSize, packet + size - 2 - optionsSize); // actual options size
-				size += m_SendBuffer.Get (packet + size, m_MTU); // payload
-				m_LocalDestination.GetOwner ()->Sign (packet, size, signature);
+				else
+				{	
+					if (isOfflineSignature)
+					{
+						const auto& offlineSignature = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetOfflineSignature ();
+						memcpy (packet + size, offlineSignature.data (), offlineSignature.size ());
+						size += offlineSignature.size (); // offline signature
+					}
+					uint8_t * signature = packet + size; // set it later
+					memset (signature, 0, signatureLen); // zeroes for now
+					size += signatureLen; // signature
+					htobe16buf (optionsSize, packet + size - 2 - optionsSize); // actual options size
+					size += m_SendBuffer.Get (packet + size, m_MTU); // payload
+					m_LocalDestination.GetOwner ()->Sign (packet, size, signature);
+				}	
 			}
 			else
 			{
 				// follow on packet
-				htobuf16 (packet + size, 0);
-				size += 2; // flags
-				htobuf16 (packet + size, 0); // no options
+				if (m_IsJavaClient && (!m_LastACKRequestTime || ts - m_LastACKRequestTime > m_MinRTT / 10))
+				{
+					m_LastACKRequestTime = ts;
+					htobe16buf (packet + size, PACKET_FLAG_DELAY_REQUESTED);
+					size += 2; // flags
+					htobe16buf (packet + size, 2); // 2 bytes delay interval
+					htobe16buf (packet + size + 2, 0); // set immediate ack interval
+					size += 2;
+				}
+				else
+				{
+					htobuf16 (packet + size, 0);
+					size += 2; // flags
+					htobuf16 (packet + size, 0); // no options
+				}
 				size += 2; // options size
 				size += m_SendBuffer.Get(packet + size, m_MTU); // payload
 			}
@@ -861,6 +1048,7 @@ namespace stream
 		}
 		if (m_SendBuffer.GetSize() == 0) m_IsBufferEmpty = true;
 		else m_IsBufferEmpty = false;
+		int numPackets = packets.size ();
 		if (packets.size () > 0)
 		{
 			if (m_SavedPackets.empty ()) // no NACKS
@@ -877,7 +1065,12 @@ namespace stream
 			}
 			SendPackets (packets);
 			m_LastSendTime = ts;
-			m_IsSendTime = false;
+			m_IsSendTime = false;			
+			if (m_RoutingSession)
+			{
+				int numSentPackets = m_RoutingSession->NumSentPackets ();
+				m_RoutingSession->SetNumSentPackets (numSentPackets + numPackets);
+			}
 			if (m_Status == eStreamStatusClosing && m_SendBuffer.IsEmpty ())
 				SendClose ();
 			if (isEmpty)
@@ -954,7 +1147,7 @@ namespace stream
 		htobe32buf (packet + size, lastReceivedSeqn);
 		size += 4; // ack Through
 		uint8_t numNacks = 0;
-		bool choking = false;
+		bool choking = m_IsChoking2;
 		if (lastReceivedSeqn > m_LastReceivedSequenceNumber)
 		{
 			// fill NACKs
@@ -1004,7 +1197,10 @@ namespace stream
 		if (choking || requestImmediateAck)
 		{
 			htobe16buf (packet + size, 2); // 2 bytes delay interval
-			htobe16buf (packet + size + 2, choking ? DELAY_CHOKING : 0); // set choking or immediate ack interval
+			if (m_IsChoking2)
+				htobe16buf (packet + size + 2, DELAY_CHOKING_2); // set choking2
+			else
+				htobe16buf (packet + size + 2, choking ? DELAY_CHOKING : 0); // set choking or immediate ack interval
 			size += 2;
 			if (requestImmediateAck) // ack request sent
 			{
@@ -1020,6 +1216,7 @@ namespace stream
 		SendPackets (std::vector<Packet *> { &p });
 		m_LastACKSendTime = ts; // for limit inbound speed
 		m_LastConfirmedReceivedSequenceNumber = lastReceivedSeqn; // for limit inbound speed
+		m_IsChoking2 = false;
 		LogPrint (eLogDebug, "Streaming: Quick Ack sent. ", (int)numNacks, " NACKs");
 	}
 
@@ -1032,28 +1229,34 @@ namespace stream
 		size += 4; // sendStreamID
 		memset (packet + size, 0, 14);
 		size += 14; // all zeroes
-		uint16_t flags = PACKET_FLAG_ECHO | PACKET_FLAG_SIGNATURE_INCLUDED | PACKET_FLAG_FROM_INCLUDED;
+		uint16_t flags = PACKET_FLAG_ECHO | PACKET_FLAG_FROM_INCLUDED;
+		if (!m_DontSign) flags |= PACKET_FLAG_SIGNATURE_INCLUDED;
 		bool isOfflineSignature = m_LocalDestination.GetOwner ()->GetPrivateKeys ().IsOfflineSignature ();
 		if (isOfflineSignature) flags |= PACKET_FLAG_OFFLINE_SIGNATURE;
 		htobe16buf (packet + size, flags);
 		size += 2; // flags
 		size_t identityLen = m_LocalDestination.GetOwner ()->GetIdentity ()->GetFullLen ();
-		size_t signatureLen = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetSignatureLen ();
 		uint8_t * optionsSize = packet + size; // set options size later
 		size += 2; // options size
 		m_LocalDestination.GetOwner ()->GetIdentity ()->ToBuffer (packet + size, identityLen);
 		size += identityLen; // from
-		if (isOfflineSignature)
-		{
-			const auto& offlineSignature = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetOfflineSignature ();
-			memcpy (packet + size, offlineSignature.data (), offlineSignature.size ());
-			size += offlineSignature.size (); // offline signature
-		}
-		uint8_t * signature = packet + size; // set it later
-		memset (signature, 0, signatureLen); // zeroes for now
-		size += signatureLen; // signature
-		htobe16buf (optionsSize, packet + size - 2 - optionsSize); // actual options size
-		m_LocalDestination.GetOwner ()->Sign (packet, size, signature);
+		if (m_DontSign)
+			htobe16buf (optionsSize, packet + size - 2 - optionsSize); // actual options size
+		else
+		{	
+			if (isOfflineSignature)
+			{
+				const auto& offlineSignature = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetOfflineSignature ();
+				memcpy (packet + size, offlineSignature.data (), offlineSignature.size ());
+				size += offlineSignature.size (); // offline signature
+			}
+			size_t signatureLen = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetSignatureLen ();
+			uint8_t * signature = packet + size; // set it later
+			memset (signature, 0, signatureLen); // zeroes for now
+			size += signatureLen; // signature
+			htobe16buf (optionsSize, packet + size - 2 - optionsSize); // actual options size
+			m_LocalDestination.GetOwner ()->Sign (packet, size, signature);
+		}	
 		p.len = size;
 		SendPackets (std::vector<Packet *> { &p });
 		LogPrint (eLogDebug, "Streaming: Ping of ", p.len, " bytes sent");
@@ -1107,18 +1310,41 @@ namespace stream
 		size++; // NACK count
 		packet[size] = 0;
 		size++; // resend delay
-		htobe16buf (packet + size, PACKET_FLAG_CLOSE | PACKET_FLAG_SIGNATURE_INCLUDED);
+		uint16_t flags = PACKET_FLAG_CLOSE;
+		if (!m_DontSign) flags |= PACKET_FLAG_SIGNATURE_INCLUDED;
+		bool isOfflineSignature = m_LocalDestination.GetOwner ()->GetPrivateKeys ().IsOfflineSignature ();
+		if (isOfflineSignature) flags |= PACKET_FLAG_OFFLINE_SIGNATURE;
+		htobe16buf (packet + size, flags);
 		size += 2; // flags
-		size_t signatureLen = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetSignatureLen ();
-		htobe16buf (packet + size, signatureLen); // signature only
-		size += 2; // options size
-		uint8_t * signature = packet + size;
-		memset (packet + size, 0, signatureLen);
-		size += signatureLen; // signature
-		m_LocalDestination.GetOwner ()->Sign (packet, size, signature);
+		if (m_DontSign)
+		{
+			memset (packet + size, 0, 2); // no options
+			size += 2; // options size
+		}	
+		else
+		{	
+			if (isOfflineSignature)
+			{
+				const auto& offlineSignature = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetOfflineSignature ();
+				memcpy (packet + size, offlineSignature.data (), offlineSignature.size ());
+				size += offlineSignature.size (); // offline signature
+			}
+			size_t signatureLen = m_LocalDestination.GetOwner ()->GetPrivateKeys ().GetSignatureLen ();
+			htobe16buf (packet + size, signatureLen); // signature only
+			size += 2; // options size
+			uint8_t * signature = packet + size;
+			memset (packet + size, 0, signatureLen);
+			size += signatureLen; // signature
+			m_LocalDestination.GetOwner ()->Sign (packet, size, signature);
+		}	
 
 		p->len = size;
 		boost::asio::post (m_Service, std::bind (&Stream::SendPacket, shared_from_this (), p));
+		if (m_RoutingSession)
+		{
+			int numSentPackets = m_RoutingSession->NumSentPackets ();
+			m_RoutingSession->SetNumSentPackets (numSentPackets + 1);
+		}
 		LogPrint (eLogDebug, "Streaming: FIN sent, sSID=", m_SendStreamID);
 	}
 
@@ -1333,12 +1559,12 @@ namespace stream
 					m_NumPacketsToSend = 1; m_PacingTimeRem = 0;
 				}	
 				m_IsSendTime = true;
-				if (m_WindowIncCounter && (m_WindowSize < MAX_WINDOW_SIZE || m_WindowDropTargetSize) && !m_SendBuffer.IsEmpty () && m_PacingTime > m_MinPacingTime)
+				if (m_WindowIncCounter && (m_WindowSize < m_MaxWindowSize || m_WindowDropTargetSize) && !m_SendBuffer.IsEmpty () && m_PacingTime > m_MinPacingTime)
 				{
 					float winSize = m_WindowSize;
 					if (m_WindowDropTargetSize)
 						winSize = m_WindowDropTargetSize;
-					float maxWinSize = MAX_WINDOW_SIZE;
+					float maxWinSize = m_MaxWindowSize;
 					if (m_LastWindowIncTime)
 						maxWinSize = (ts - m_LastWindowIncTime) / (m_RTT / MAX_WINDOW_SIZE_INC_PER_RTT) + winSize;
 					for (int i = 0; i < m_NumPacketsToSend; i++)
@@ -1353,7 +1579,7 @@ namespace stream
 									m_WindowDropTargetSize += (m_WindowDropTargetSize - (m_LastWindowDropSize - PREV_SPEED_KEEP_TIME_COEFF)) / m_WindowDropTargetSize; // some magic here
 								else
 									m_WindowDropTargetSize += (m_WindowDropTargetSize - (1 - PREV_SPEED_KEEP_TIME_COEFF)) / m_WindowDropTargetSize;
-								if (m_WindowDropTargetSize > MAX_WINDOW_SIZE) m_WindowDropTargetSize = MAX_WINDOW_SIZE;
+								if (m_WindowDropTargetSize > m_MaxWindowSize) m_WindowDropTargetSize = m_MaxWindowSize;
 								m_WindowIncCounter--;
 								if (m_WindowDropTargetSize >= maxWinSize)
 								{
@@ -1369,7 +1595,7 @@ namespace stream
 									m_WindowSize += (m_WindowSize - (m_LastWindowDropSize - PREV_SPEED_KEEP_TIME_COEFF)) / m_WindowSize; // some magic here
 								else
 									m_WindowSize += (m_WindowSize - (1 - PREV_SPEED_KEEP_TIME_COEFF)) / m_WindowSize;
-								if (m_WindowSize > MAX_WINDOW_SIZE) m_WindowSize = MAX_WINDOW_SIZE;
+								if (m_WindowSize > m_MaxWindowSize) m_WindowSize = m_MaxWindowSize;
 								m_WindowIncCounter--;
 								if (m_WindowSize >= maxWinSize)
 								{
@@ -1384,7 +1610,7 @@ namespace stream
 					UpdatePacingTime ();
 				}
 				m_LastWindowIncTime = ts;
-				if (m_IsNAcked || m_IsResendNeeded || m_IsClientChoked) // resend packets
+				if (m_IsNAcked || m_IsResendNeeded || m_IsClientChoked || m_IsClientChoked2) // resend packets
 					ResendPacket ();
 				else if (m_WindowSize > int(m_SentPackets.size ())) // send packets
 					SendBuffer ();
@@ -1443,7 +1669,7 @@ namespace stream
 		// collect packets to resend
 		auto ts = i2p::util::GetMillisecondsSinceEpoch ();
 		std::vector<Packet *> packets;
-		if (m_IsNAcked)
+		if (m_IsNAcked && !m_IsClientChoked && !m_IsClientChoked2)
 		{
 			for (auto it : m_NACKedPackets)
 			{
@@ -1471,6 +1697,8 @@ namespace stream
 						it->resent = false;
 					it->sendTime = ts;
 					packets.push_back (it);
+					if (m_IsClientChoked2 && it->GetSeqn () == m_DropWindowDelaySequenceNumber)
+						m_IsClientChoked2 = false;
 					if ((int)packets.size () >= m_NumPacketsToSend) break;
 				}
 			}
@@ -1491,7 +1719,7 @@ namespace stream
 						m_LastWindowDropSize = m_WindowDropTargetSize;
 					else
 						m_LastWindowDropSize = m_WindowSize;
-					m_WindowDropTargetSize = m_LastWindowDropSize * 0.75; // -25% to drain queue
+					m_WindowDropTargetSize = m_LastWindowDropSize * 0.5; // -50% to drain queue
 					if (m_WindowDropTargetSize < MIN_WINDOW_SIZE)
 						m_WindowDropTargetSize = MIN_WINDOW_SIZE;
 					m_WindowIncCounter = 0; // disable window growth
@@ -1503,7 +1731,6 @@ namespace stream
 			}
 			else if (m_IsTimeOutResend)
 			{
-				m_IsTimeOutResend = false;
 				m_RTO = INITIAL_RTO; // drop RTO to initial upon tunnels pair change
 				m_WindowDropTargetSize = INITIAL_WINDOW_SIZE;
 				m_LastWindowDropSize = 0;
@@ -1534,12 +1761,12 @@ namespace stream
 			SendPackets (packets);
 			m_LastSendTime = ts;
 			m_IsSendTime = false;
-			if (m_IsNAcked || m_IsResendNeeded || m_IsClientChoked) ScheduleSend ();
 		}
-		else if (!m_IsClientChoked)
+		else if (!m_IsClientChoked && !m_IsClientChoked2)
 			SendBuffer ();
-		if (!m_IsNAcked && !m_IsResendNeeded) ScheduleResend ();
-		if (m_IsClientChoked) ScheduleSend ();
+		m_IsSendTime = false;
+		if (m_IsTimeOutResend) ScheduleResend (); // ^ m_IsTimeOutResend = false
+		else if (m_IsNAcked || m_IsResendNeeded || m_IsClientChoked || m_IsClientChoked2) ScheduleSend ();
 	}
 
 	void Stream::ScheduleAck (int timeout)
@@ -1590,19 +1817,29 @@ namespace stream
 			auto remoteLeaseSet = m_LocalDestination.GetOwner ()->FindLeaseSet (m_RemoteIdentity->GetIdentHash ());
 			if (!remoteLeaseSet)
 			{
-				LogPrint (eLogWarning, "Streaming: LeaseSet ", m_RemoteIdentity->GetIdentHash ().ToBase64 (), m_RemoteLeaseSet ? " expired" : " not found");
+				LogPrint (eLogWarning, "Streaming: LeaseSet ", m_RemoteIdentity->GetIdentHash ().ToBase32 (), m_RemoteLeaseSet ? " expired" : " not found");
 				if (!m_IsIncoming) // outgoing
 				{	
+					auto requestCallback = [s = shared_from_this ()](std::shared_ptr<i2p::data::LeaseSet> ls)
+					    {
+							if (!ls && s->m_Status == eStreamStatusOpen) // LeaseSet not found
+							{
+								// close the socket without sending FIN or RST
+								s->m_Status = eStreamStatusClosed;
+								s->AsyncClose ();
+							}	
+						};
+					
 					if (m_RemoteLeaseSet && m_RemoteLeaseSet->IsPublishedEncrypted ())
 					{
 						m_LocalDestination.GetOwner ()->RequestDestinationWithEncryptedLeaseSet (
-							std::make_shared<i2p::data::BlindedPublicKey>(m_RemoteIdentity));
+							std::make_shared<i2p::data::BlindedPublicKey>(m_RemoteIdentity), requestCallback);
 						return; // we keep m_RemoteLeaseSet for possible next request
 					}
 					else
 					{
 						m_RemoteLeaseSet = nullptr;
-						m_LocalDestination.GetOwner ()->RequestDestination (m_RemoteIdentity->GetIdentHash ()); // try to request for a next attempt
+						m_LocalDestination.GetOwner ()->RequestDestination (m_RemoteIdentity->GetIdentHash (), requestCallback); // try to request for a next attempt	
 					}
 				}
 				else // incoming
@@ -1710,7 +1947,7 @@ namespace stream
 			m_LastWindowDropSize = m_WindowDropTargetSize * ((m_MinRTT + m_Jitter*4) / m_FastRTT);
 		else
 			m_LastWindowDropSize = m_WindowSize * ((m_MinRTT + m_Jitter*4) / m_FastRTT);
-		m_WindowDropTargetSize = m_LastWindowDropSize * 0.75; // -25% to drain queue
+		m_WindowDropTargetSize = m_LastWindowDropSize * 0.5; // -50% to drain queue
 		if (m_WindowDropTargetSize < MIN_WINDOW_SIZE)
 			m_WindowDropTargetSize = MIN_WINDOW_SIZE;
 		m_WindowIncCounter = 0; // disable window growth
@@ -1749,7 +1986,7 @@ namespace stream
 	}
 
 	StreamingDestination::StreamingDestination (std::shared_ptr<i2p::client::ClientDestination> owner, uint16_t localPort, bool gzip):
-		m_Owner (owner), m_LocalPort (localPort), m_Gzip (gzip),
+		m_Owner (owner), m_LocalPort (localPort), m_Gzip (gzip), 
 		m_PendingIncomingTimer (m_Owner->GetService ()), 
 		m_LastCleanupTime (i2p::util::GetSecondsSinceEpoch ())
 	{
@@ -1772,6 +2009,7 @@ namespace stream
 	void StreamingDestination::Stop ()
 	{
 		ResetAcceptor ();
+		ResetPongHandler ();
 		m_PendingIncomingTimer.cancel ();
 		m_PendingIncomingStreams.clear ();
 		{
@@ -1818,6 +2056,8 @@ namespace stream
 			{
 				// pong
 				LogPrint (eLogInfo, "Streaming: Pong received rSID=", packet->GetReceiveStreamID ());
+				if (m_PongHandler != nullptr)
+					m_PongHandler (packet->from ? packet->from->GetDestinationPtr () : nullptr);
 				DeletePacket (packet);
 				return;
 			}
@@ -1997,6 +2237,16 @@ namespace stream
 		m_Acceptor = nullptr;
 	}
 
+	void StreamingDestination::SetPongHandler (const PongHandler& handler)
+	{
+		m_PongHandler = handler;
+	}
+		
+	void StreamingDestination::ResetPongHandler ()
+	{
+		m_PongHandler = nullptr;
+	}
+		
 	void StreamingDestination::AcceptOnce (const Acceptor& acceptor)
 	{
 		boost::asio::post (m_Owner->GetService (), [acceptor, this](void)
@@ -2053,14 +2303,18 @@ namespace stream
 		}
 	}
 
-	void StreamingDestination::HandleDataMessagePayload (const uint8_t * buf, size_t len)
+	void StreamingDestination::HandleDataMessagePayload (const uint8_t * buf, size_t len,
+		i2p::garlic::ECIESX25519AEADRatchetSession * from)
 	{
 		// unzip it
 		Packet * uncompressed = NewPacket ();
 		uncompressed->offset = 0;
 		uncompressed->len = m_Inflator.Inflate (buf, len, uncompressed->buf, MAX_PACKET_SIZE);
 		if (uncompressed->len)
+		{
+			uncompressed->from = from;
 			HandleNextPacket (uncompressed);
+		}	
 		else
 			DeletePacket (uncompressed);
 	}
