@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2020, The PurpleI2P Project
+* Copyright (c) 2013-2024, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -51,12 +51,13 @@ namespace util
 			MemoryPool (): m_Head (nullptr) {}
 			~MemoryPool ()
 			{
-				while (m_Head)
-				{
-					auto tmp = m_Head;
-					m_Head = static_cast<T*>(*(void * *)m_Head); // next
-					::operator delete ((void *)tmp);
-				}
+				CleanUp ();
+			}
+
+			void CleanUp ()
+			{
+				CleanUp (m_Head);
+				m_Head = nullptr;
 			}
 
 			template<typename... TArgs>
@@ -93,13 +94,25 @@ namespace util
 					std::bind (&MemoryPool<T>::Release, this, std::placeholders::_1));
 			}
 
+		protected:
+
+			void CleanUp (T * head)
+			{
+				while (head)
+				{
+					auto tmp = head;
+					head = static_cast<T*>(*(void * *)head); // next
+					::operator delete ((void *)tmp);
+				}
+			}
+
 		protected:
 
 			T * m_Head;
 	};
 
 	template<class T>
-	class MemoryPoolMt: public MemoryPool<T>
+	class MemoryPoolMt: private MemoryPool<T>
 	{
 		public:
 
@@ -118,6 +131,14 @@ namespace util
 				this->Release (t);
 			}
 
+			void ReleaseMt (T * * arr, size_t num)
+			{
+				if (!arr || !num) return;
+				std::lock_guard<std::mutex> l(m_Mutex);
+				for (size_t i = 0; i < num; i++)
+					this->Release (arr[i]);
+			}
+
 			template<template<typename, typename...>class C, typename... R>
 			void ReleaseMt(const C<T *, R...>& c)
 			{
@@ -126,6 +147,24 @@ namespace util
 					this->Release (it);
 			}
 
+			template<typename... TArgs>
+			std::shared_ptr<T> AcquireSharedMt (TArgs&&... args)
+			{
+				return std::shared_ptr<T>(AcquireMt (std::forward<TArgs>(args)...),
+					std::bind<void (MemoryPoolMt<T>::*)(T *)> (&MemoryPoolMt<T>::ReleaseMt, this, std::placeholders::_1));
+			}
+
+			void CleanUpMt ()
+			{
+				T * head;
+				{
+					std::lock_guard<std::mutex> l(m_Mutex);
+					head = this->m_Head;
+					this->m_Head = nullptr;
+				}
+				if (head) this->CleanUp (head);
+			}
+
 		private:
 
 			std::mutex m_Mutex;
@@ -138,12 +177,14 @@ namespace util
 			RunnableService (const std::string& name): m_Name (name), m_IsRunning (false) {}
 			virtual ~RunnableService () {}
 
-			boost::asio::io_service& GetIOService () { return m_Service; }
+			auto& GetIOService () { return m_Service; }
 			bool IsRunning () const { return m_IsRunning; };
 
 			void StartIOService ();
 			void StopIOService ();
 
+			void SetName (std::string_view name);
+
 		private:
 
 			void Run ();
@@ -153,7 +194,7 @@ namespace util
 			std::string m_Name;
 			volatile bool m_IsRunning;
 			std::unique_ptr<std::thread> m_Thread;
-			boost::asio::io_service m_Service;
+			boost::asio::io_context m_Service;
 	};
 
 	class RunnableServiceWithWork: public RunnableService
@@ -161,11 +202,11 @@ namespace util
 		protected:
 
 			RunnableServiceWithWork (const std::string& name):
-				RunnableService (name), m_Work (GetIOService ()) {}
+				RunnableService (name), m_Work (GetIOService ().get_executor ()) {}
 
 		private:
 
-			boost::asio::io_service::work m_Work;
+			boost::asio::executor_work_guard<boost::asio::io_context::executor_type> m_Work;
 	};
 
 	void SetThreadName (const char *name);
@@ -187,11 +228,13 @@ namespace util
 	namespace net
 	{
 		int GetMTU (const boost::asio::ip::address& localAddress);
+		int GetMaxMTU (const boost::asio::ip::address_v6& localAddress); // check tunnel broker for ipv6 address
 		const boost::asio::ip::address GetInterfaceAddress (const std::string & ifname, bool ipv6=false);
 		boost::asio::ip::address_v6 GetYggdrasilAddress ();
 		bool IsLocalAddress (const boost::asio::ip::address& addr);
 		bool IsInReservedRange (const boost::asio::ip::address& host);
 		bool IsYggdrasilAddress (const boost::asio::ip::address& addr);
+		bool IsPortInReservedRange (const uint16_t port) noexcept;
 	}
 }
 }
