@@ -343,19 +343,28 @@ namespace client
 		}
 	}
 
-	void BOBCommandSession::SendReplyOK (const char * msg)
+	void BOBCommandSession::SendReplyOK (std::string_view msg)
 	{
 		std::ostream os(&m_SendBuffer);
 		os << "OK";
-		if(msg)
-		{
+		if (!msg.empty ())
 			os << " " << msg;
-		}
 		os << std::endl;
 		Send ();
-	}
-
-	void BOBCommandSession::SendReplyError (const char * msg)
+	}	
+		
+	void BOBCommandSession::SendReplyOK (const std::vector<std::string_view>& strings)
+	{
+		std::ostream os(&m_SendBuffer);
+		os << "OK";
+		if (!strings.empty ()) os << " ";
+		for (auto& it: strings)
+			os << it;
+		os << std::endl;
+		Send ();
+	}	
+		
+	void BOBCommandSession::SendReplyError (std::string_view msg)
 	{
 		std::ostream os(&m_SendBuffer);
 		os << "ERROR " << msg << std::endl;
@@ -366,10 +375,10 @@ namespace client
 	{
 		std::ostream os(&m_SendBuffer);
 		os << "BOB 00.00.10" << std::endl;
-		SendReplyOK();
+		SendReplyOK(std::string_view()); // empty string
 	}
 
-	void BOBCommandSession::SendRaw (const char * data)
+	void BOBCommandSession::SendRaw (std::string_view data)
 	{
 		std::ostream os(&m_SendBuffer);
 		os << data << std::endl;
@@ -564,9 +573,7 @@ namespace client
 			if (!dest)
 			{
 				m_Nickname = operand;
-				std::string msg ("Nickname set to ");
-				msg += m_Nickname;
-				SendReplyOK (msg.c_str ());
+				SendReplyOK ({ "Nickname set to ", m_Nickname });
 			}
 			else
 				SendReplyError ("tunnel is active");
@@ -591,11 +598,7 @@ namespace client
 				m_Nickname = operand;
 			}
 			if (m_Nickname == operand)
-			{
-				std::string msg ("Nickname set to ");
-				msg += m_Nickname;
-				SendReplyOK (msg.c_str ());
-			}
+				SendReplyOK ({"Nickname set to ", m_Nickname});
 			else
 				SendReplyError ("no nickname has been set");
 		}
@@ -606,7 +609,7 @@ namespace client
 	void BOBCommandSession::NewkeysCommandHandler (const char * operand, size_t len)
 	{
 		LogPrint (eLogDebug, "BOB: newkeys");
-		i2p::data::SigningKeyType signatureType = i2p::data::SIGNING_KEY_TYPE_DSA_SHA1;
+		i2p::data::SigningKeyType signatureType = i2p::data::SIGNING_KEY_TYPE_EDDSA_SHA512_ED25519;
 		i2p::data::CryptoKeyType cryptoType = i2p::data::CRYPTO_KEY_TYPE_ELGAMAL;
 		if (*operand)
 		{
@@ -628,14 +631,14 @@ namespace client
 
 
 		m_Keys = i2p::data::PrivateKeys::CreateRandomKeys (signatureType, cryptoType, true);
-		SendReplyOK (m_Keys.GetPublic ()->ToBase64 ().c_str ());
+		SendReplyOK (m_Keys.GetPublic ()->ToBase64 ());
 	}
 
 	void BOBCommandSession::SetkeysCommandHandler (const char * operand, size_t len)
 	{
 		LogPrint (eLogDebug, "BOB: setkeys ", operand);
 		if (*operand && m_Keys.FromBase64 (operand))
-			SendReplyOK (m_Keys.GetPublic ()->ToBase64 ().c_str ());
+			SendReplyOK (m_Keys.GetPublic ()->ToBase64 ());
 		else
 			SendReplyError ("invalid keys");
 	}
@@ -644,7 +647,7 @@ namespace client
 	{
 		LogPrint (eLogDebug, "BOB: getkeys");
 		if (m_Keys.GetPublic ()) // keys are set ?
-			SendReplyOK (m_Keys.ToBase64 ().c_str ());
+			SendReplyOK (m_Keys.ToBase64 ());
 		else
 			SendReplyError ("keys are not set");
 	}
@@ -653,7 +656,7 @@ namespace client
 	{
 		LogPrint (eLogDebug, "BOB: getdest");
 		if (m_Keys.GetPublic ()) // keys are set ?
-			SendReplyOK (m_Keys.GetPublic ()->ToBase64 ().c_str ());
+			SendReplyOK (m_Keys.GetPublic ()->ToBase64 ());
 		else
 			SendReplyError ("keys are not set");
 	}
@@ -759,7 +762,7 @@ namespace client
 				auto leaseSet = localDestination->FindLeaseSet (addr->identHash);
 				if (leaseSet)
 				{
-					SendReplyOK (leaseSet->GetIdentity ()->ToBase64 ().c_str ());
+					SendReplyOK (leaseSet->GetIdentity ()->ToBase64 ());
 					return;
 				}
 			}
@@ -768,7 +771,7 @@ namespace client
 			auto requstCallback = [s](std::shared_ptr<i2p::data::LeaseSet> ls)
 				{
 					if (ls)
-						s->SendReplyOK (ls->GetIdentity ()->ToBase64 ().c_str ());
+						s->SendReplyOK (ls->GetIdentity ()->ToBase64 ());
 					else
 						s->SendReplyError ("LeaseSet Not found");
 				};
@@ -794,7 +797,7 @@ namespace client
 			}
 			auto ls = i2p::data::netdb.FindLeaseSet (addr->identHash);
 			if (ls)
-				SendReplyOK (ls->GetIdentity ()->ToBase64 ().c_str ());
+				SendReplyOK (ls->GetIdentity ()->ToBase64 ());
 			else
 				SendReplyError ("Local LeaseSet Not found");
 		}
@@ -802,6 +805,94 @@ namespace client
 			SendReplyError ("empty lookup address");
 	}
 
+	void BOBCommandSession::PingCommandHandler (const char * operand, size_t len)
+	{
+		LogPrint (eLogDebug, "BOB: ping ", operand);
+		if (*operand)
+		{
+			auto addr = context.GetAddressBook ().GetAddress (operand);
+			if (!addr)
+			{
+				SendReplyError ("Address Not found");
+				return;
+			}
+			auto localDestination = (m_CurrentDestination && m_CurrentDestination->IsRunning ()) ? 
+				m_CurrentDestination->GetLocalDestination () : i2p::client::context.GetSharedLocalDestination ();
+			if (!localDestination)
+			{
+				SendReplyError ("No local destination");
+				return;
+			}
+			if (addr->IsIdentHash ())
+			{
+				// we might have leaseset already
+				auto leaseSet = localDestination->FindLeaseSet (addr->identHash);
+				if (leaseSet)
+				{	
+					boost::asio::post (localDestination->GetService (), 
+						std::bind (&BOBCommandSession::SendPing, shared_from_this (), leaseSet));
+					return;
+				}
+			}
+			// request LeaseSet
+			if (addr->IsIdentHash ())
+				localDestination->RequestDestination (addr->identHash,
+					std::bind (&BOBCommandSession::SendPing, shared_from_this (), std::placeholders::_1));
+			else
+				localDestination->RequestDestinationWithEncryptedLeaseSet (addr->blindedPublicKey,
+					std::bind (&BOBCommandSession::SendPing, shared_from_this (), std::placeholders::_1));
+		}
+		else
+			SendReplyError ("empty ping address");
+	}	
+
+	void BOBCommandSession::SendPing (std::shared_ptr<const i2p::data::LeaseSet> ls)
+	{
+		if (!ls)
+		{
+			SendReplyError ("LeaseSet Not found");
+			return;
+		}	
+		auto localDestination = (m_CurrentDestination && m_CurrentDestination->IsRunning ()) ? 
+				m_CurrentDestination->GetLocalDestination () : i2p::client::context.GetSharedLocalDestination ();
+		if (!localDestination)
+		{
+			SendReplyError ("No local destination");
+			return;
+		}
+		auto streamingDestination = localDestination->GetStreamingDestination ();
+		if (!streamingDestination)
+		{
+			SendReplyError ("No streaming destination");
+			return;
+		}
+		auto timer = std::make_shared<boost::asio::deadline_timer>(localDestination->GetService ());
+		timer->expires_from_now (boost::posix_time::milliseconds(BOB_PING_TIMEOUT));
+		timer->async_wait ([streamingDestination, s = shared_from_this ()](const boost::system::error_code& ecode)
+		{
+			if (ecode != boost::asio::error::operation_aborted)
+			{
+				LogPrint (eLogDebug, "BOB: Pong not received after ", BOB_PING_TIMEOUT, " millliseconds");
+				streamingDestination->ResetPongHandler ();
+				s->SendReplyError ("timeout");
+			}
+		});
+		auto ts = i2p::util::GetMillisecondsSinceEpoch ();
+		streamingDestination->SetPongHandler (
+		    [streamingDestination, timer, ts, s = shared_from_this ()](const i2p::data::IdentHash * from)
+			{
+				int t = i2p::util::GetMillisecondsSinceEpoch () - ts;
+				if (t < 0) t = 0;
+				streamingDestination->ResetPongHandler ();
+				timer->cancel ();
+				if (from) 
+					s->SendReplyOK ({"pong ", "from ", from->ToBase32(), ".b32.i2p: time=", std::to_string (t), " ms"});
+				else
+					s->SendReplyOK ({"pong: time=", std::to_string (t), " ms"});         
+			});
+		streamingDestination->SendPing (ls);
+	}	
+		
 	void BOBCommandSession::ClearCommandHandler (const char * operand, size_t len)
 	{
 		LogPrint (eLogDebug, "BOB: clear");
@@ -819,7 +910,7 @@ namespace client
 		for (const auto& it: destinations)
 		{
 			BuildStatusLine(false, it.second, statusLine);
-			SendRaw(statusLine.c_str());
+			SendRaw(statusLine);
 			if(m_Nickname.compare(it.second->GetNickname()) == 0)
 				sentCurrent = true;
 		}
@@ -828,7 +919,7 @@ namespace client
 			// add the current tunnel to the list.
 			// this is for the incomplete tunnel which has not been started yet.
 			BuildStatusLine(true, m_CurrentDestination, statusLine);
-			SendRaw(statusLine.c_str());
+			SendRaw(statusLine);
 		}
 		SendReplyOK ("Listing done");
 	}
@@ -839,14 +930,10 @@ namespace client
 		const char * value = strchr (operand, '=');
 		if (value)
 		{
-			std::string msg ("option ");
 			*(const_cast<char *>(value)) = 0;
 			m_Options[operand] = value + 1;
-			msg += operand;
+			SendReplyOK ({ "option ", operand, " set to ", value + 1 });
 			*(const_cast<char *>(value)) = '=';
-			msg += " set to ";
-			msg += value + 1;
-			SendReplyOK (msg.c_str ());
 		}
 		else
 			SendReplyError ("malformed");
@@ -864,7 +951,7 @@ namespace client
 		{
 			// tunnel destination exists
 			BuildStatusLine(false, dest, statusLine);
-			SendReplyOK(statusLine.c_str());
+			SendReplyOK(statusLine);
 		}
 		else
 		{
@@ -872,7 +959,7 @@ namespace client
 			{
 				// tunnel is incomplete / has not been started yet
 				BuildStatusLine(true, nullptr, statusLine);
-				SendReplyOK(statusLine.c_str());
+				SendReplyOK(statusLine);
 			}
 			else
 			{
@@ -891,15 +978,14 @@ namespace client
 			{
 				ss << " " << x.first;
 			}
-			const std::string &str = ss.str();
-			SendReplyOK(str.c_str());
+			SendReplyOK(ss.str ());
 		}
 		else
 		{
 			auto it = helpStrings.find(operand);
 			if (it != helpStrings.end ())
 			{
-				SendReplyOK(it->second.c_str());
+				SendReplyOK(it->second);
 				return;
 			}
 			SendReplyError("No such command");
@@ -949,6 +1035,7 @@ namespace client
 		m_CommandHandlers[BOB_COMMAND_QUIET] = &BOBCommandSession::QuietCommandHandler;
 		m_CommandHandlers[BOB_COMMAND_LOOKUP] = &BOBCommandSession::LookupCommandHandler;
 		m_CommandHandlers[BOB_COMMAND_LOOKUP_LOCAL] = &BOBCommandSession::LookupLocalCommandHandler;
+		m_CommandHandlers[BOB_COMMAND_PING] = &BOBCommandSession::PingCommandHandler;	
 		m_CommandHandlers[BOB_COMMAND_CLEAR] = &BOBCommandSession::ClearCommandHandler;
 		m_CommandHandlers[BOB_COMMAND_LIST] = &BOBCommandSession::ListCommandHandler;
 		m_CommandHandlers[BOB_COMMAND_OPTION] = &BOBCommandSession::OptionCommandHandler;
