@@ -81,14 +81,26 @@ namespace client
 		return ourIP;
 	}
 
+	boost::asio::ip::address GetLoopbackAddress6For(const i2p::data::IdentHash & addr)
+	{
+		boost::asio::ip::address_v6::bytes_type bytes;
+		const uint8_t * ident = addr;
+		bytes[0] = 0xfd;
+		memcpy (bytes.data ()+1, ident, 15);
+		boost::asio::ip::address ourIP = boost::asio::ip::address_v6 (bytes);
+		return ourIP;
+	}
+		
 #ifdef __linux__
-	static void MapToLoopback(std::shared_ptr<boost::asio::ip::tcp::socket> sock, const i2p::data::IdentHash & addr)
+	static void MapToLoopback(std::shared_ptr<boost::asio::ip::tcp::socket> sock, const i2p::data::IdentHash & addr, bool isV4)
 	{
 		if (sock)
 		{
-			// bind to 127.x.x.x address
+			// bind to 127.x.x.x address for ipv4
 			// where x.x.x are first three bytes from ident
-			auto ourIP = GetLoopbackAddressFor(addr);
+			// bind to fdxx.xxxx.xxxx.xxxx.xxxx.xxxx.xxxx.xxxx address for ipv6
+			// where xx.xxxx.xxxx.xxxx.xxxx.xxxx.xxxx.xxxx are first 15 bytes from ident
+			auto ourIP = isV4 ? GetLoopbackAddressFor(addr) : GetLoopbackAddress6For(addr);
 			boost::system::error_code ec;
 			sock->bind (boost::asio::ip::tcp::endpoint (ourIP, 0), ec);
 			if (ec)
@@ -103,12 +115,19 @@ namespace client
 		{
 			I2PTunnelSetSocketOptions (m_Socket);
 #ifdef __linux__
-			if (isUniqueLocal && m_RemoteEndpoint.address ().is_v4 () &&
-				m_RemoteEndpoint.address ().to_v4 ().to_bytes ()[0] == 127)
-			{
-				m_Socket->open (boost::asio::ip::tcp::v4 ());
+			if (isUniqueLocal && m_RemoteEndpoint.address ().is_loopback ())
+			{	
 				auto ident = m_Stream->GetRemoteIdentity()->GetIdentHash();
-				MapToLoopback(m_Socket, ident);
+				if (m_RemoteEndpoint.address ().is_v4 ())
+				{
+					m_Socket->open (boost::asio::ip::tcp::v4 ());
+					MapToLoopback(m_Socket, ident, true);
+				}
+				else if (m_RemoteEndpoint.address ().is_v6 ())
+				{
+					m_Socket->open (boost::asio::ip::tcp::v6 ());
+					MapToLoopback(m_Socket, ident, false);
+				}	
 			}
 #endif
 			m_Socket->async_connect (m_RemoteEndpoint, std::bind (&I2PTunnelConnection::HandleConnect,
@@ -428,7 +447,7 @@ namespace client
 					else
 					{
 						// strip up some headers
-						static const std::array<std::string_view, 2> excluded // list of excluded headers
+						static constexpr std::array<std::string_view, 2> excluded // list of excluded headers
 						{
 							"Keep-Alive:", "X-I2P"
 						};
@@ -511,7 +530,7 @@ namespace client
 					if (line == "\r") endOfHeader = true;
 					else
 					{
-						static const std::array<std::string_view, 5> excluded // list of excluded headers
+						static constexpr std::array<std::string_view, 5> excluded // list of excluded headers
 						{
 							"Server:", "Date:", "X-Runtime:", "X-Powered-By:", "Proxy"
 						};
@@ -738,17 +757,10 @@ namespace client
 		boost::system::error_code ec;
 		auto addr = boost::asio::ip::make_address (m_Address, ec);
 		if (!ec)
-		{
 			m_Endpoint.address (addr);
-			Accept ();
-		}
 		else
-		{
-			auto resolver = std::make_shared<boost::asio::ip::tcp::resolver>(GetService ());
-			resolver->async_resolve (m_Address, "",
-				std::bind (&I2PServerTunnel::HandleResolve, this,
-					std::placeholders::_1, std::placeholders::_2, resolver));
-		}
+			Resolve (nullptr);
+		Accept ();
 	}
 
 	void I2PServerTunnel::Stop ()
@@ -758,13 +770,26 @@ namespace client
 		auto localDestination = GetLocalDestination ();
 		if (localDestination)
 			localDestination->StopAcceptingStreams ();
-
+		if (m_Resolver)	
+			m_Resolver->cancel ();
+		
 		ClearHandlers ();
 	}
 
-	void I2PServerTunnel::HandleResolve (const boost::system::error_code& ecode, boost::asio::ip::tcp::resolver::results_type endpoints,
-		std::shared_ptr<boost::asio::ip::tcp::resolver> resolver)
+	bool I2PServerTunnel::Resolve (std::shared_ptr<i2p::stream::Stream> stream)
 	{
+		if (m_Resolver) return false; // already resolving
+		m_Resolver = std::make_shared<boost::asio::ip::tcp::resolver>(GetService ());
+		m_Resolver->async_resolve (m_Address, "",
+			std::bind (&I2PServerTunnel::HandleResolve, this,
+				std::placeholders::_1, std::placeholders::_2, stream));
+		return true;
+	}	
+		
+	void I2PServerTunnel::HandleResolve (const boost::system::error_code& ecode, boost::asio::ip::tcp::resolver::results_type endpoints,
+		std::shared_ptr<i2p::stream::Stream> stream)
+	{
+		m_Resolver = nullptr;
 		if (!ecode)
 		{
 			bool found = false;
@@ -808,7 +833,8 @@ namespace client
 			auto addr = ep.address ();
 			LogPrint (eLogInfo, "I2PTunnel: Server tunnel ", (*endpoints.begin ()).host_name (), " has been resolved to ", addr);
 			m_Endpoint.address (addr);
-			Accept ();
+			if (stream)
+				Connect (stream);
 		}
 		else
 			LogPrint (eLogError, "I2PTunnel: Unable to resolve server tunnel address ", m_Address, ": ", ecode.message ());
@@ -869,16 +895,28 @@ namespace client
 					return;
 				}
 			}
-			// new connection
-			auto conn = CreateI2PConnection (stream);
-			AddHandler (conn);
-			if (m_LocalAddress)
-				conn->Connect (*m_LocalAddress);
-			else
-				conn->Connect (m_IsUniqueLocal);
+			if (!m_Endpoint.address ().is_unspecified ())
+				Connect (stream);
+			else if (!Resolve (stream))
+			{
+				LogPrint (eLogWarning, "I2PTunnel: Address ", m_Address, " can't be resolved. Incoming connection dropped");
+				stream->Close ();
+				return;
+			}	
 		}
 	}
 
+	void I2PServerTunnel::Connect (std::shared_ptr<i2p::stream::Stream> stream)
+	{
+		// new connection
+		auto conn = CreateI2PConnection (stream);
+		AddHandler (conn);
+		if (m_LocalAddress)
+			conn->Connect (*m_LocalAddress);
+		else
+			conn->Connect (m_IsUniqueLocal);
+	}	
+		
 	std::shared_ptr<I2PTunnelConnection> I2PServerTunnel::CreateI2PConnection (std::shared_ptr<i2p::stream::Stream> stream)
 	{
 		return std::make_shared<I2PTunnelConnection> (this, stream, GetEndpoint (), m_SSLCtx);
